#!/usr/bin/python2

# PyLogger (pylogger.py)
#
# Blogging Tool written in Python and distributed under the
# terms of the LGPL.
#
# Written by A.Sajjad Zaidi (http://www.sajjadzaidi.com/)
# Copyright April 2003
#

import os,sys
import string
from string import atoi
import re
import shutil
import processWeb
from filefuncts import openTemplate
from filefuncts import getConfig
from filefuncts import getTimes
from clilogger import parseShellCmd
from clilogger import promptForData



def parseCGIForm():
	"""Parse CGI Form and display main page if no arguments given"""

	import cgi
	from cgilogger import printMainPage
	from cgilogger import printPreviewPage

	form = cgi.FieldStorage()

	if form.has_key('cancel'):
		printMainPage()
		sys.exit(0)
	elif form.has_key('preview'):
		blSubject = form['subject'].value
		blBody = form['blogbody'].value
		printPreviewPage(blSubject, blBody)
	elif form.has_key('confirm'):
		blSubject = form['subject'].value
		blBody = form['blogbody'].value
		blBody = string.replace(blBody,'\n','<br />\n')
		blBody = string.replace(blBody,'\r','')
		addNewEntry(blSubject, blBody)
		printMainPage()
	else:
		printMainPage()
		sys.exit(0)
	return()





def addNewEntry(blogSubject, blogBody):
	"""Get input from somewhere and insert it in the main page, together with
	the time stamp. Create new page if it doesn't exist."""

	allTimes = getTimes('newblog')

	blogDate = allTimes[0]
	blogTime = allTimes[1]
	blogMonth = string.lower(allTimes[2])
	blogYear = allTimes[3]

	# Check if a pylogger index file already exists
	if os.path.exists('index.pyl'):
		try:
			fp = open('index.pyl')
			fullFile = string.join(fp.readlines(),'')
			fp.close()
		except:
			print 'Error opening index.pyl'
	# Create a new file
	else:
		fullFile = ''

	datePat = re.compile(r'<!-- NEW ENTRIES START HERE -->\s*<p></p><h5>(.*?)</h5>\s*<div>\s*<a name="(\d+)">')
	allMatches = datePat.search(fullFile)
	if allMatches:
		lastBlogDate = allMatches.group(1)
		lastBookMark = allMatches.group(2)
	else:
		lastBlogDate = ''
		lastBookMark = '100000'

	if atoi(lastBookMark) < 100000:
		newBookMark = atoi(lastBookMark) + 100001
	else:
		newBookMark = atoi(lastBookMark) + 1

	blogSection = openTemplate('newbloghtml.txt', {
'blogDate' : blogDate,
'newBookMark' : newBookMark,
'blogSubject' : blogSubject,
'blogBody' : blogBody,
'blogTime' : blogTime,
'newBookMark' : newBookMark,
'blogYear' : blogYear,
'blogMonth' : blogMonth
})

	if lastBlogDate == blogDate:
		updatedText = string.replace(fullFile,'\t\t\t\t\t\t\t\t<!-- NEW ENTRIES START HERE -->\n', '')
		updatedText = string.replace(updatedText,'<p></p><h5>%s</h5>' % blogDate, blogSection)
	elif lastBlogDate == '':
		updatedText = blogSection
	else:
		updatedText = string.replace(fullFile,'<!-- NEW ENTRIES START HERE -->', blogSection)

	try:
		w = open('index.pyl', 'w')
		w.write(updatedText)
		w.close()
	except:
		print 'Error writing index.pyl'
		sys.exit(1)

	updateIndexFile(newBookMark,blogDate,blogTime,blogSubject)

	return()



def updateIndexFile(indexNumber,fullDate,blogTime,blogSubject):
	"""Append an entry for this blog into the index file. This
	will work great with a database"""

	indexFile = getConfig('blogindex')

	if os.path.exists(indexFile):
		fp = open(indexFile)
		oldDataList = fp.readlines()
		fp.close()
		oldData = string.join(oldDataList,'')
	else:
		oldData = ''

	i = open(indexFile,'w')
	i.write(oldData)
	i.write('%s,"%s","%s","%s"\n' %(indexNumber,fullDate,blogTime,blogSubject))
	i.close()
	return()




def generateXML(xmlFileName, xmlType):
	"""Generate XML from the last 20 entries"""

	localRoot = getConfig('localroot')

	xmlHeader = openTemplate('xmlheader.txt', {})
	xmlFooter = openTemplate('xmlfooter.txt', {})

	if xmlType == 'short':
		rssPat = re.compile(r'<p><b>([^<]+):</b><br></br>\s*(.{50,200}\b).*?<p class="blogend">Posted at (\d\d:\d\d\w\w) <a href="([^"]*)">Comments[^<]*</a>[^<]*<a href="([^"]*)"')
	else:
		rssPat = re.compile(r'<p><b>([^<]+):</b><br></br>\s*(\S.*?)<p class="blogend">Posted at (\d\d:\d\d\w\w) <a href="([^"]*)">Comments[^<]*</a>[^<]*<a href="([^"]*)"')

	fp = open('index.pyl')
	indexFile = string.join(fp.readlines(), '')
	fp.close()

	indexFile = string.replace(indexFile,'\t','')
	indexFile = string.replace(indexFile,'\n',' ')

	rssMatches = rssPat.findall(indexFile)

	xmlFullPath = '%s/%s' % (localRoot, xmlFileName)

	n = open(xmlFullPath, 'w')

	n.write(xmlHeader)
	n.write('\n')

	count = 20

	for m in rssMatches:
		if count > 0:
			blogTitle = m[0]
			blogLink = 'http://www.sajjadzaidi.com%s' % m[4]
			commentLink = 'http://www.sajjadzaidi.com%s' % m[3]
			blogTime = m[2]
			if xmlType == 'short':
				blogBody = '<p>%s ....' % m[1]
			else:
				blogBody = '<p>%s' % m[1]
			itemText = openTemplate('xmlitem.txt', {
'blogtitle' : blogTitle,
'bloglink' : blogLink,
'blogbody' : blogBody,
'blogauthor' : 'sajjad@zaidi.jp',
'blogcategory' : 'Religion',
'commentlink' : commentLink,
'permalink' : blogLink,
'blogtime' : blogTime
})
			itemText = string.replace(itemText, '\r', '')
			n.write(itemText)
			count = count - 1
		else:
			break

	n.write(xmlFooter)

	n.close()
	return()





def pingBlogURL():
	"""Ping blog URL after we update (usually weblogs.com)"""

	pingURL = getConfig('pingurl')
	runWget = os.popen('wget -q -O - -U "Pylogger" "%s"' % pingURL)

	pingOutput = string.join(runWget.readlines(),'')
	runWget.close()
	print pingOutput

	return()





###########################################
##                 Start                 ##
###########################################



testRoot = getConfig('testroot')


# Store the template in string to be used later
os.chdir('%s' % testRoot)

# Check how we were accessesd
if os.environ.has_key("GATEWAY_INTERFACE"):
	parseCGIForm()
elif len(sys.argv) > 1:
	blogSubject, blCategory, blogBody = parseShellCmd()
	addNewEntry(blogSubject, blogBody)
else:
	blogSubject, blCategory, blogBody = promptForData()
	addNewEntry(blogSubject, blogBody)


# Generate the final templates
processWeb.tmplGen()


# Generate XML feed from index.pyl
shortFeed = getConfig('shortxmlfeed')
if shortFeed != '':
	generateXML(shortFeed, 'short')

fullFeed = getConfig('fullxmlfeed')
if fullFeed != '':
	generateXML(fullFeed, 'full')

commentsFeed = getConfig('commentsxmlfeed')
if commentsFeed != '':
	generateXML(commentsFeed, 'comments')

# Final update to website
#processWeb.runRsync()

#allowPing = getConfig('allowping')
#if allowPing == 1:
#	pingBlogURL()
#
sys.exit(0)

